FName = 'RA_16ch_25kHz_1min';
load(FName); 

sps = srate;    %25000Hz
NChannels = nbchan;
%%
%Let's plot 16 first seconds of this record
Bt = data(:,(1:(sps*16))); %16 sec to plot
Shift = 200; %uV, introduced just for nice plotting, the first channel is at the top 
Delta = (0:nbchan)*Shift;
Bt1 = zeros(size(Bt));
for I_t = 1:nbchan
  Bt1(I_t,:) = Bt(I_t,:)-Delta(I_t);  
end 
figure(1); clf;
X = (1:size(Bt1,2))/sps;
plot(X,Bt1(:,:)');
ylim([-Shift-Delta(nbchan) Shift]);
xlim([0 X(end)]);
xlabel('Time (s)');
title('Original data, 16 channels');
%%
%As first three chanels don't have spikes - remove them and plot again
Bt = data(4:end,(1:(sps*16))); %16 sec to plot
nbchan = size(Bt,1);
Shift = 200; %uV, introduced just for nice plotting, the first channel is at the top 
Delta = (0:nbchan)*Shift;
Bt1 = zeros(size(Bt));
for I_t = 1:nbchan
  Bt1(I_t,:) = Bt(I_t,:)-Delta(I_t);  
end 
figure(2); clf;
X = (1:size(Bt1,2))/sps;
plot(X,Bt1(:,:)');
ylim([-Shift-Delta(nbchan) Shift]);
xlim([0 X(end)]);
xlabel('Time (s)');
title('Original data, channels 4:16');

%%
%get 800-5000 Hz filtered signal
%minimal filter length 32000/800 = 40, 
%80 is bad 800 is good
MaxF = sps/2;
b1 = fir1(800,[800 5000]/MaxF);    %Buzsaki tetrodes
%b1 = fir1(800,[400 3000]/MaxF);    %default WaveClus
%Lets look at this filter first
figure(3); clf;
freqz(b1,1,500:1:5500, sps);
%%
%b1 = fir1(800,[300 3000]/MaxF);
%b1 = fir1(800,[300]/MaxF,'High');
Bt = data(4:end,:)';
tic
if gpuDeviceCount>0
  Btt = filtfiltGPU(b1,1,Bt);     %1.022435 seconds GTX580 single precision 
else  
  Btt = filtfilt(b1,1,double(Bt));     % 6.764905 seconds i7 6x4GHz                       
end  
%Btt = filtfiltGPU2(b1,1,Bt);     %2xGTX580 good for large arrays only  
toc 
%Btt = Bt; %trial - filtration is Off
Bf = Btt';
%Let's compute median from modulus to scale thrershold in sigmas
MyMedian = median(abs(Btt));
%Let's get sigma estimate from Median:
MySigma = MyMedian/0.6745;
%%
%Now we plot small amount of data
Bft = Bf(:,(1:(sps*16))); %16 sec to plot
Shift = 150; %uV, introduced just for nice plotting, the first channel is at the top 
Delta = (0:13)*Shift;
Bfs = zeros(size(Bft));
for I_t = 1:13
  Bfs(I_t,:) = Bft(I_t,:)-Delta(I_t);  
end 
%
figure(4); clf;
X = (1:size(Bfs,2))/sps*1;
plot(X,Bfs(:,:)');
ylim([-150-Delta(13) 150]);
xlim([0 X(end)]);
xlabel('Time (s)');
title('800-5000 Hz filtered data');
%%
%now we compute power in 0.2 ms window; Buzsaki had 0.2, 

Span2 = 2;%for +/-0.1 ms
Span = 2*Span2+1;  %round(sps*1e-3);   
P = zeros(size(Bf));
Bf2 = (Bf.*Bf)';
%for J_t = (Span2+1):(size(Bf,2)-Span2) %WE REPLACE THIS WITH "filter"
%  P(:,J_t) = sum(Bf2(:,(J_t-Span2):(J_t+Span2)))/Span;  
%end
bAverage = ones(Span,1)/Span;
Pt = filter(bAverage,1,Bf2); %1.422957 seconds, for larger arrays can be done in GPU
Pt = Pt';
P(:,(Span2+1):(end-Span2)) = sqrt(Pt(:,Span:end));
%%
%let's try relative threshold
Threshold = MySigma*8;  %was 4
Shift = 75; %uV, introduced just for nice plotting, the first channel is at the top 
Delta = (0:13)*Shift;
Ps = zeros(size(Bft));
for I_t = 1:13
  Ps(I_t,:) = P(I_t,1:size(Bft,2))-Delta(I_t);  
end 
figure(5); clf;
X = (1:size(Bft,2))/sps*1;
plot(X,Ps(:,:)');
ylim([-Delta(13) Shift]);
xlim([0 X(end)]);
for I_t = 1:13
  line([0 X(end)],[1 1]*Threshold(I_t)-Delta(I_t),'color','k');  
end    
xlabel('Time (s)');
title('Sqrt(power) in 0.25 ms sliding window');
%%
%Let's compute array of boolean indicators where spikes are
BA = false(size(P));
for I_t = 1:13
  BA(I_t,:) = (P(I_t,:)-Threshold(I_t)>=0);  
end
BAA = any(BA,1);
%now compute intervals where spikes can be between as transitions 0->1 and
%1->0
SpikeIntervals = zeros(length(BAA)/10,2);   %/10 - approximate, 
Counter = 0;
for I_t = 2:length(BAA)
  if (~BAA(I_t-1)) && BAA(I_t)
    Counter = Counter+1;
    SpikeIntervals(Counter,1) = I_t;
  end
  if BAA(I_t-1) && (~BAA(I_t))
    SpikeIntervals(Counter,2) = I_t-1;
  end        
end   
SpikeIntervals((Counter+1):end,:) = [];
NSpikes = Counter;
disp(['Spikes: ' int2str(NSpikes)]);
%%
%%Let's find a precize location of spike where its amplitude is max in the
%best channel, take range over threshold +/- 0.5 ms - +/-12 counts
SpikeDelta = 4; %counts - neglect spikes if distance between them <0.16 ms
SpikeLocation = zeros(NSpikes,1);
SpikeChannel = SpikeLocation;
Counter = 0;
for I_t = 1:NSpikes  
  X = (SpikeIntervals(I_t,1)-12):(SpikeIntervals(I_t,2)+12);
  Y = abs(Bf(:,X));
  [Value Ind] = max(Y,[],2);
  [Value2 Channel] = max(Value);
  SpikeLocation_tmp = SpikeIntervals(I_t,1)-12+Ind(Channel)-1; 
  if (Bf(Channel,SpikeLocation_tmp)<0) %take only negative spikes
    if (I_t==1) || ((I_t>1) && (abs(SpikeLocation(Counter)-SpikeLocation_tmp)>SpikeDelta))
      Counter = Counter+1;
      SpikeLocation(Counter) = SpikeLocation_tmp;
      SpikeChannel(Counter) = Channel;
    end  
  end    
end 
SpikeLocation((Counter+1):end) = []; 
SpikeChannel((Counter+1):end) = [];
NSpikes = Counter;
disp(['Spikes w/o duplicates, negative: ' int2str(NSpikes)]); %297/336/373; 12067/15614
%%
%resample 25 kHz 4x to get 100 kHz, smooth, align and downsample to 25 kHz

%Now take spike location +/- (40 counts + Filter orderspan+reserve);
Delta = 26 + 32 + 10;
Spikes = zeros(NSpikes,nbchan,1+2*Delta);
for I_t = 1:NSpikes
  Spikes(I_t,:,:) = Bf(:,(SpikeLocation(I_t)-Delta):(SpikeLocation(I_t)+Delta));  
end    
%
Frame = 1;
figure(6); clf;
%Let's plot 64 spikes
NHorizontal = 5;
NVertical = 5;
NTotal = NHorizontal*NVertical;
for I_t = 1:NTotal 
  subplot(NHorizontal,NVertical,I_t);
  plot((-13:13)/25,squeeze(Spikes(NTotal*(Frame-1)+I_t,:,Delta+1+(-13:13)))');
  xlim([-13 13]/25);
  ylim([-120 70]);
  box off
end
%%
%make 4x array
NRep = 4;
Spikes4 = zeros(size(Spikes,1),size(Spikes,2),size(Spikes,3)*NRep);
for I_t = 1:NRep
  Spikes4(:,:,I_t:4:end) = Spikes;  
end    
%low-pass filter it at 5 kHz
MaxF2 = sps*NRep/2;
%%
%b3 = fir1(50*NRep,14877/MaxF2); %default - low-pass
%b3 = fir1(32*NRep,14241/MaxF2); %default - low-pass
b3 = fir1(32*NRep,25/32*14241/MaxF2); %default - low-pass
figure(7); clf;
freqz(b3,1,4000:1:20000, sps*NRep); %just to cut frequencies above Nyquist
%%
Spikes4f = zeros(size(Spikes4));
NGPUs = gpuDeviceCount; 
%matlabpool(NGPUs); 
matlabpool; %it is better matlabpool 12 for 6 cores
tic
parfor I_t = 1:nbchan    
  S = squeeze(Spikes4(:,I_t,:)); 
  if gpuDeviceCount>0  
    gpuDevice(mod(I_t,NGPUs)+1);
    Spikes4f(:,I_t,:) = filtfiltGPUs1(b3,1,S')';
  else
    Spikes4f(:,I_t,:) = filtfilt(b3,1,double(S'))'; 
  end    
  disp(num2str(I_t));
end
toc     %GPU: 2.392616 seconds - 12 labs
matlabpool close
%%
Frame = 1;
figure(8); clf;
%Let's plot 64 spikes
NHorizontal = 5;
NVertical = 5;
NTotal = NHorizontal*NVertical;
for I_t = 1:NTotal 
  subplot(NHorizontal,NVertical,I_t);
  plot((-(13*NRep):(13*NRep))/(25*NRep),squeeze(Spikes4f(NTotal*(Frame-1)+I_t,:,NRep*Delta+1+1+((-13*NRep):(13*NRep))))');
  xlim([-13 13]/25);
  ylim([-120 70]);
  box off
end
%%
%Now shift
Width2 = 13;    %1/2 of the window width
SpikeShapes = zeros(size(Spikes,1),size(Spikes,2),1+2*Width2);  %+/-0.5 ms
MiddlePoint = floor(size(Spikes4f,3)/2)+1;  %248
for I_t = 1:size(Spikes,1)  %episodes
    [Value Pos] = min(Spikes4f(I_t,SpikeChannel(I_t),(MiddlePoint-12):(MiddlePoint+12)));  
    AbsPos = MiddlePoint-12+Pos-1;
    V = (AbsPos-Width2*4):4:(AbsPos+Width2*4);
    SpikeShapes(I_t,:,:) = Spikes4f(I_t,:,V);
end 
%%
Frame = 1;
figure(9); clf;
%Let's plot 64 spikes
NHorizontal = 8;
NVertical = 8;
NTotal = NHorizontal*NVertical;
for I_t = 1:NTotal 
  subplot(NHorizontal,NVertical,I_t);
  plot((-Width2:Width2)/25,squeeze(SpikeShapes(NTotal*(Frame-1)+I_t,:,:))');
  xlim([-Width2 Width2]/25);
  ylim([-120 70]);
  box off
end
%%
%%prepare for saving and save
SpikeShapes = single(SpikeShapes);
SpikeChannel = uint8(SpikeChannel);
SpikeLocation = uint32(SpikeLocation);
save([FName '_SpikeShapes_1ms'],'SpikeShapes','SpikeLocation','SpikeChannel','sps','FName');



